package plugins;

import java.util.ArrayList;
import java.util.List;


public class Plugin {
	private String name;
	private String description;
	private String command;
	private String platform;
	
	private List<Switch> switches;
		

	public Plugin(String name, String desc, String comm, String platform) {
		this.name = name;
		this.description = desc;
		this.command = comm;
		this.platform = platform;
		this.switches = new ArrayList<Switch>();
	}
	
	public Plugin() {
		this (null, null, null, null);		
	}

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public String getDescription() {
		return description;
	}

	public void setDescription(String description) {
		this.description = description;
	}

	public String getCommand() {
		return command;
	}

	public void setCommand(String command) {
		this.command = command;
	}

	public String getPlatform() {
		return platform;
	}

	public void setPlatform(String platform) {
		this.platform = platform;
	}
	
	public List<Switch> getSwitches() {
		return switches;
	}
	public Switch getSwitches(int i) {
		return switches.get(i);
	}

	public void setSwitches(List<Switch> switches) {
		this.switches = switches;
	}
	
	public void addSwitch(String shortName, String longName, String desc, boolean req) {
		this.addSwitch(new Switch(shortName, longName, desc, req));
	}
	
	public void addSwitch(Switch sw) {
		this.switches.add(sw);
	}
	
	public void clearSwitches() {
		this.switches.clear();
	}
	
	public void removeSwitch(String longName) {
		for (Switch sw: switches) {
			if (sw.longName.equalsIgnoreCase(longName))
				this.removeSwitch(sw);
		}
	}
	public void removeSwitch(Switch sw) {
		this.switches.remove(sw);
	}
	
	public String getXml() {
		StringBuilder sb = new StringBuilder();
		sb.append("	<plugin name=")
			.append("\"" + this.name + "\" ")
			.append("description=" + "\"" + this.description + "\" ")
			.append("command=" + "\"" + this.command + "\" ")
			.append("platform=" + "\"" + this.platform + "\">\n");
		if (this.getSwitches() != null) {
			for (Switch sw : this.switches) {
				sb.append("		<switch ")
					.append("shortName=" + "\"" + sw.getShortName() + "\" ")
					.append("longName=" + "\"" + sw.getLongName() + "\" ")
					.append("description=" + "\"" + sw.getDescription() + "\" ")
					.append("requireValue=" + "\"" + String.valueOf(sw.isRequireArgument()) + "\"/>\n");
			}
		}
		sb.append("	</plugin>\n");
		return new String(sb);
	}
	
	public String toString() {
		StringBuilder sb = new StringBuilder();
		sb.append("Plugin: ").append(this.getName() + "\n")
			.append("Description: ").append(this.getDescription() + "\n")
			.append("Command: ").append(this.getCommand() + "\n")
			.append("Platform: ").append(this.getPlatform() + "\n");
			
		if (this.getSwitches() != null && this.getSwitches().size() != 0) {
			sb.append("Switches: \n");
			for (Switch sw : this.getSwitches()) {
				sb.append("-" + sw.getShortName() + " ")
					.append("--" + sw.getLongName());
				if (sw.isRequireArgument())
					sb.append(" <arg>:");
				else
					sb.append(" :");
				sb.append(sw.getDescription() + "\n");
			}
		}
		return sb.toString();
	}
	
	
}
