package plugins;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/*
 * Command format: command [switch1 [arg1] switch2 [arg2]...] inputfile outputfile
 */

public class PluginExecutor {
	private List<String> command;

	private StringBuilder stdout, stderr;

	private boolean exec;

	public PluginExecutor() {
		command = new ArrayList<String>();
		stderr = new StringBuilder();
		stdout = new StringBuilder();
		this.setExecutable(false);
	}
		
	/**
	 * 
	 * @param command	- cesta k vykonatelnemu suboru
	 * @param argMap	- mapa argumentov alebo null ak nie su ziadne argumenty
	 * @param inFile	- vstupny subor
	 * @param outFile	- vystupny subor
	 * @return	- 0 ak ok, -1 ak error
	 */
	/**
*
* @param command - cesta k vykonatelnemu suboru
* @param argMap - mapa argumentov alebo null ak nie su ziadne argumenty
* @param inFile - vstupny subor
* @param outFile - vystupny subor
* @return - 0 ak ok, -1 ak error
*/
public int setCommand(String command, Map<String, String> argMap, String inFile, String outFile, boolean windows) {

if (command == null || inFile == null || outFile == null)
return -1;

this.setExecutable(false);
this.command.clear();

if (windows) {
this.command.add("cmd.exe");
this.command.add("/c");
}

this.command.add(command);

if (argMap != null) {
for (String sw : argMap.keySet()) {
String arg = argMap.get(sw);

this.command.add(this.checkSwitch(sw));
if (arg != null)
this.command.add(arg);
}
}
this.command.add(inFile);
this.command.add(outFile);

this.setExecutable(true);

return 0;
}

	private String checkSwitch(String sw) {
		StringBuilder sb = new StringBuilder();

		if (!sw.startsWith("-")) {
			if (sw.length() > 1)
				sb.append("--");
			else
				sb.append("-");

			sb.append(sw);
			return sb.toString();
		}
		return sw;
	}

	private boolean isExecutable() {
		return exec;
	}

	private void setExecutable(boolean exec) {
		this.exec = exec;
	}

	public String getStdErr() {
		return this.stderr.toString();
	}
	
	public String getStdout() {
		return this.stdout.toString();
	}

	public int execute() {
		if (!this.isExecutable()) return -1;

		ProcessBuilder pb;
		final Process process;
		InputStream is;
		
		this.stderr = new StringBuilder();
		this.stdout = new StringBuilder();

		try {
			pb = new ProcessBuilder(command);
			process = pb.start();

			is = process.getErrorStream();
			this.readStd(is, stderr);
			
			is = process.getInputStream();
			this.readStd(is, stdout);

		} catch (IOException e) {
			this.stderr.append("Could not execute command:\n")
				.append(e.getMessage());
			if (e.getMessage().contains("Permission denied"))
				this.stderr.append("\nPerhaps execute bit is not set?\n");
		}
		return 0;
	}
	
	private void readStd(InputStream is, StringBuilder sb) throws IOException {
		InputStreamReader isr;
		BufferedReader br;
		
		isr = new InputStreamReader(is);
		br = new BufferedReader(isr);
		
		String line;
		while ((line = br.readLine()) != null)
			sb.append(line + "\n");
	}

	public String toString() {
		return this.command.toString();
	}

	
}
