package plugins;

import GUIcore.TypOs;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.List;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import org.openide.util.Exceptions;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

public final class PluginManager {

    private List<Plugin> plugins;
    private DocumentBuilderFactory docBuilderFactory;
    private DocumentBuilder docBuilder;
    private Document document;
    String CESTA;
    private File file;


    public PluginManager() {
        this.plugins = new ArrayList<Plugin>();
        try {
            TypOs poww=new TypOs();
                   CESTA=poww.getCesta();
            file= new File(CESTA+"plugins.xml");
            loadPlugins();
           Copy();
        } catch (SAXException ex) {
            Exceptions.printStackTrace(ex);
        } catch (IOException ex) {
            Exceptions.printStackTrace(ex);
        }
    }

    public List<Plugin> getPlugins() {
        return plugins;
    }

    public int size() {
        return this.getPlugins().size();
    }

    public Plugin getPluginAt(int index) {
        if (index < 0 || index > (this.getPlugins().size() - 1)) {
            return null;
        }
        return this.getPlugins().get(index);
    }

    public Plugin getPluginByName(String name) {
        for (Plugin p : this.getPlugins()) {
            if (p.getName().equals(name)) {
                return p;
            }
        }
        return null;
    }

    public int addPlugin(Plugin plugin) {
        if (plugin == null) {
            return -1;
        }
        this.getPlugins().add(plugin);
        this.savePlugins();
        return 0;
    }

    /**
     * Funkcia na pridanie pluginu
     *
     * @param name		- meno pluginu
     * @param desc		- opis
     * @param comm		- cesta k vykonatelnemu suboru
     * @param platform  - platforma
     * @param switches	- zoznam prepinacov alebo null ak sa prepinace budu zadavat manualne;
     */
    public int addPlugin(String name, String desc, String comm, String platform, List<Switch> switches) {
        if (name == null || desc == null || comm == null || platform == null) {
            return -1;
        }

        Plugin plugin = new Plugin(name, desc, comm, platform);
        if (switches != null) {
            plugin.setSwitches(switches);
        }

        this.addPlugin(plugin);
        return this.getPlugins().indexOf(plugin);
    }

    /**
     * Funkcia na modifikovanie pluginu
     *
     * Zadavaju sa hodnoty novych parametrov alebo null ak sa parameter nemeni
     *
     * @param index		- index pluginu v zozname
     * @param name		- meno pluginu
     * @param desc		- opis
     * @param comm		- cesta k vykonatelnemu suboru
     * @param platform  - platforma
     * @param switches	- zoznam prepinacov;
     */
    public int modifyPluginAt(int index, String name, String desc, String comm, String platform, List<Switch> switches) {
        if (index < 0 || index > (this.getPlugins().size() - 1)) {
            return -1;
        }

        Plugin plugin = this.getPluginAt(index);
        if (plugin == null) {
            return -1;
        }

        if (name != null) {
            plugin.setName(name);
        }
        if (desc != null) {
            plugin.setDescription(desc);
        }
        if (comm != null) {
            plugin.setCommand(comm);
        }
        if (platform != null) {
            plugin.setPlatform(platform);
        }
        if (switches != null) {
            plugin.setSwitches(switches);
        }

        this.savePlugins();
        return 0;
    }

    /**
     * Funkcia na modifikovanie pluginu
     *
     * Zadavaju sa hodnoty novych parametrov alebo null ak sa parameter nemeni
     *
     * @param plugin	- konkretny plugin, ktory sa uz nachadza v zozname pluginov
     * @param name		- meno pluginu
     * @param desc		- opis
     * @param comm		- cesta k vykonatelnemu suboru
     * @param platform  - platforma
     * @param switches	- zoznam prepinacov;
     */
    public int modifyPlugin(Plugin plugin, String name, String desc, String comm, String platform, List<Switch> switches) {
        if (plugin == null || !this.getPlugins().contains(plugin)) {
            return -1;
        }

        this.modifyPluginAt(this.getPlugins().indexOf(plugin), name, desc, comm, platform, switches);

        return 0;
    }

    public int removePlugin(Plugin plugin) {
        if (plugin == null) {
            return -1;
        }
        this.getPlugins().remove(plugin);
        this.savePlugins();
        return 0;
    }

    public int removePluginAt(int index) {
        if (index < 0 || index > (this.getPlugins().size() - 1)) {
            return -1;
        }
        this.removePlugin(this.getPlugins().remove(index));
        return 0;
    }

    private int loadPlugins() throws SAXException, IOException {
        Plugin plugin;
        Element pluginElement;
        Switch sw;
        NodeList plugins, switches;
        NamedNodeMap plAttr, swAttr;
        this.docBuilderFactory = DocumentBuilderFactory.newInstance();
        try {
            this.docBuilder = docBuilderFactory.newDocumentBuilder();
        } catch (ParserConfigurationException ex) {
            Exceptions.printStackTrace(ex);
        }

        if ((file.exists()) && (file.length() != 0)) {
            document = docBuilder.parse(file);
        } else {
            document = docBuilder.parse(getResource());
        }

        document.getDocumentElement().normalize();

        plugins = document.getElementsByTagName("plugin");
        for (int i = 0; i < plugins.getLength(); i++) {
            plugin = new Plugin();

            plAttr = plugins.item(i).getAttributes();
            if (plAttr.getLength() < 4) {
                return -1;
            }

            for (int k = 0; k < plAttr.getLength(); k++) {
                String tag = plAttr.item(k).getNodeName();
                String value = plAttr.item(k).getTextContent();

                if (tag.equals("name")) {
                    plugin.setName(value);
                }
                if (tag.equals("description")) {
                    plugin.setDescription(value);
                }
                if (tag.equals("command")) {
                    plugin.setCommand(value);
                }
                if (tag.equals("platform")) {
                    plugin.setPlatform(value);
                }
            }

            pluginElement = (Element) plugins.item(i);
            switches = pluginElement.getElementsByTagName("switch");

            for (int j = 0; j < switches.getLength(); j++) {
                sw = new Switch();

                swAttr = switches.item(j).getAttributes();
                if (swAttr.getLength() < 4) {
                    return -1;
                }

                for (int k = 0; k < swAttr.getLength(); k++) {
                    String tag = swAttr.item(k).getNodeName();
                    String value = swAttr.item(k).getTextContent();

                    if (tag.equals("shortName")) {
                        sw.setShortName(value);
                    }
                    if (tag.equals("longName")) {
                        sw.setLongName(value);
                    }
                    if (tag.equals("requireValue")) {
                        sw.setRequireArgument(value);
                    }
                    if (tag.equals("description")) {
                        sw.setDescription(value);
                    }
                }
                plugin.addSwitch(sw);
            }
            this.getPlugins().add(plugin);
        }
        return 0;
    }

    private void savePlugins() {
        String head = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n\n<!-- author, description, xml informations, etc. -->\n\n<plugins>\n";
        String tail = "</plugins>";

        try {
            file.createNewFile();
            BufferedWriter out = new BufferedWriter(new FileWriter(this.file));
            out.write(head);
            for (Plugin p : this.getPlugins()) {
                out.write(p.getXml());
            }
            out.write(tail);
            out.close();
        } catch (IOException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
    }
public void Copy() {
  
        InputStream[] ce0 = new InputStream[5];
        ce0[0] = getClass().getResourceAsStream("/resources/kissToBlif.jar");
        ce0[1] = getClass().getResourceAsStream("/resources/kissToVhdl.jar");
        ce0[2] = getClass().getResourceAsStream("/resources/plaToBlif.jar");
        ce0[3] = getClass().getResourceAsStream("/resources/plaToVhdl.jar");
        ce0[4] = getClass().getResourceAsStream("/resources/mvsis.jar");
        String []pom=new String[5];
        pom[0]="kissToBlif.jar";
        pom[1]="kissToVhdl.jar";
        pom[2]="plaToBlif.jar";
        pom[3]="plaToVhdl.jar";
        pom[4]="mvsis.jar";
        for (int i = 0; i < ce0.length; i++) {

        File f2 = new File(CESTA + pom[i]);

        if(!f2.exists()){
        OutputStream out;
        try {
            out = new FileOutputStream(f2);


            byte[] buf = new byte[1024];
            int len;
            while ((len = ce0[i].read(buf)) > 0) {
                out.write(buf, 0, len);
            }
            ce0[i].close();
            out.close();
            f2.setExecutable(true);

            Plugin poo=getPlugins().get(i);
            String ces=CESTA+ pom[i];
            modifyPlugin(poo, poo.getName(), poo.getDescription(), ces, poo.getPlatform(), poo.getSwitches());

        } catch (FileNotFoundException ex) {
        } catch (IOException e) {
        }
             //   String command = getPluginByName(pom[i]).getCommand();


    }
        }
        //savePlugins();
    }
    private InputStream getResource() {
        InputStream resource = getClass().getResourceAsStream("/resources/plugins.xml");

        return resource;
    }

   

    
}
