
package syntax;

import token.TokenConstant;
import javax.swing.text.*;
import java.awt.*;

public class SyntaxUtilities {

    public static boolean regionMatches(boolean ignoreCase, Segment text,
            int offset, String match) {
        int length = offset + match.length();
        char[] textArray = text.array;
        if (length > text.offset + text.count) {
            return false;
        }
        for (int i = offset, j = 0; i < length; i++, j++) {
            char c1 = textArray[i];
            char c2 = match.charAt(j);
            if (ignoreCase) {
                c1 = Character.toUpperCase(c1);
                c2 = Character.toUpperCase(c2);
            }
            if (c1 != c2) {
                return false;
            }
        }
        return true;
    }

    public static boolean regionMatches(boolean ignoreCase, Segment text,
            int offset, char[] match) {
        int length = offset + match.length;
        char[] textArray = text.array;
        if (length > text.offset + text.count) {
            return false;
        }
        for (int i = offset, j = 0; i < length; i++, j++) {
            char c1 = textArray[i];
            char c2 = match[j];
            if (ignoreCase) {
                c1 = Character.toUpperCase(c1);
                c2 = Character.toUpperCase(c2);
            }
            if (c1 != c2) {
                return false;
            }
        }
        return true;
    }

    public static SyntaxStyle[] getDefaultSyntaxStyles() {
        SyntaxStyle[] styles = new SyntaxStyle[TokenConstant.ID_COUNT];


        
        styles[TokenConstant.KEYWORD1] = new SyntaxStyle(Font.SANS_SERIF, 12,Color.red, 0, 1);
        styles[TokenConstant.KEYWORD2] = new SyntaxStyle(Font.SANS_SERIF, 12,Color.blue, 0, 1);
        styles[TokenConstant.KEYWORD3] = new SyntaxStyle(Font.SANS_SERIF, 12,new Color(0x009600), 1, 0);
       
        return styles;
    }

    public static int paintSyntaxLine(Segment line, TokenConstant tokens,
            SyntaxStyle[] styles, TabExpander expander, Graphics gfx,
            int x, int y) {
        Font defaultFont = gfx.getFont();
        Color defaultColor = gfx.getColor();

        int offset = 0;
        for (;;) {
            byte id = tokens.id;
            if (id == TokenConstant.END) {
                break;
            }

            int length = tokens.length;
            if (id == TokenConstant.NULL) {
                if (!defaultColor.equals(gfx.getColor())) {
                    gfx.setColor(defaultColor);
                }
                if (!defaultFont.equals(gfx.getFont())) {
                    gfx.setFont(defaultFont);
                }
            } else {
                styles[id].setGraphicsFlags(gfx, defaultFont);
            }

            line.count = length;
            x = Utilities.drawTabbedText(line, x, y, gfx, expander, 0);
            line.offset += length;
            offset += length;

            tokens = tokens.next;
        }

        return x;
    }

    // private members
    private SyntaxUtilities() {
    }
}
