/*
 * Nastavaju defaultne hodnoty pre workWindow
 * 
 */
package textArea;

import javax.swing.JPopupMenu;
import java.awt.Color;
import java.awt.Font;
import javax.swing.undo.UndoManager;
import events.DefaultInputHandler;
import events.InputHandler;
import syntax.SyntaxDocument;
import syntax.SyntaxStyle;
import syntax.SyntaxUtilities;

public class TextAreaDefaults {

    private static TextAreaDefaults DEFAULTS;
    public InputHandler inputHandler;
    public SyntaxDocument document;
    public boolean editable;
    public boolean caretVisible;
    public boolean caretBlinks;
    public boolean blockCaret;
    public int electricScroll;
    public int cols;
    public int rows;
    public SyntaxStyle[] styles;
    public Color caretColor;
    public Color selectionColor;
    public Color lineHighlightColor;
    public boolean lineHighlight;
    public Color bracketHighlightColor;
    public boolean bracketHighlight;
    public Color eolMarkerColor;
    public boolean eolMarkers;
    public boolean paintInvalid;
    public JPopupMenu popup;
    public boolean leftWindowCollapsed;
    public int leftWindowWidth;
    public Color leftWindowBgColor;
    public Color leftWindowFgColor;
    public Color leftWindowHighlightColor;
    public Color leftWindowBorderColor;
    public int leftWindowBorderWidth;
    public int leftWindowNumberAlignment;
    public Font leftWindowFont;
    public Color background;
    public Color colorFont;
    public Font painterFont;
    public UndoManager undoMgr;

    /**
     * Returns a new TextAreaDefaults object with the default values filled
     * in.
     */
    public static TextAreaDefaults getDefaults() {
        if (DEFAULTS == null) {
            DEFAULTS = new TextAreaDefaults();

            DEFAULTS.inputHandler = new DefaultInputHandler();
            DEFAULTS.inputHandler.addDefaultKeyBindings();
            DEFAULTS.document = new SyntaxDocument();
            DEFAULTS.editable = true;
            DEFAULTS.leftWindowCollapsed = false;

            DEFAULTS.leftWindowWidth = 30;
            DEFAULTS.leftWindowBgColor = new Color(170, 170, 170);
            DEFAULTS.leftWindowFgColor = Color.blue;
            DEFAULTS.leftWindowHighlightColor = new Color(71, 71, 71);
            DEFAULTS.leftWindowBorderColor = new Color(170, 170, 170);
            DEFAULTS.leftWindowBorderWidth = 5;
            DEFAULTS.leftWindowNumberAlignment = LeftWindow.RIGHT;
            DEFAULTS.leftWindowFont = new Font("monospaced", Font.BOLD, 11);


            DEFAULTS.background = Color.white;
            DEFAULTS.colorFont = Color.black;
            DEFAULTS.painterFont = new Font(Font.SANS_SERIF, Font.TYPE1_FONT, 12);
            DEFAULTS.caretVisible = true;
            DEFAULTS.caretBlinks = true;
            DEFAULTS.electricScroll = 3;

            DEFAULTS.cols = 80;
            DEFAULTS.rows = 20;
            DEFAULTS.styles = SyntaxUtilities.getDefaultSyntaxStyles();
            DEFAULTS.caretColor = Color.red;
            DEFAULTS.selectionColor = new Color(0xccccff);
            DEFAULTS.lineHighlightColor = new Color(170, 170, 170);
            DEFAULTS.lineHighlight = true;
            DEFAULTS.bracketHighlightColor = Color.black;
            DEFAULTS.bracketHighlight = true;
            DEFAULTS.eolMarkerColor = new Color(0x009999);
            DEFAULTS.eolMarkers = true;
            DEFAULTS.paintInvalid = true;

            DEFAULTS.undoMgr = new UndoManager();
            DEFAULTS.document.addUndoableEditListener(DEFAULTS.undoMgr);


        }

        return DEFAULTS;
    }
}
